is_brain = False

if is_brain:
    import Brain


def sub_train_state(epoch,sub_epoch,batch,loader):
    if is_brain:
        Brain(epoch,sub_epoch,batch,loader)
    else:
        print("Sub epoch status : " , epoch,sub_epoch,batch,loader)

def sub_train_end(epoch,sub_epoch,mean_loss):
    if is_brain:
        Brain(epoch,sub_epoch,mean_loss)
    else:
        print("Sub epoch end : ",epoch,sub_epoch,mean_loss)

def validate_state(batch,loader):
    if is_brain:
        Brain(batch,loader)
    else:
        print("Validation status : ",batch,loader)


def validate_end(epoch,mean_loss,img_auc,pix_auc):
    if is_brain:
        Brain(epoch,mean_loss,img_auc,pix_auc)
    else:
        print("Validation End : ",epoch,mean_loss,img_auc,pix_auc)

def test_state(batch,loader):
    if is_brain:
        Brain(batch,loader)
    else:
        print("Test status : ",batch,loader)

def test_end(epoch,mean_loss,img_auc,pix_auc):
    if is_brain:
        Brain(epoch,mean_loss,img_auc,pix_auc)
    else:
        print("Test End : ",epoch,mean_loss,img_auc,pix_auc)


import math
import torch


def positionalencoding2d(D, H, W):
    """
    :param D: dimension of the model
    :param H: H of the positions
    :param W: W of the positions
    :return: DxHxW position matrix
    """
    if D % 4 != 0:
        raise ValueError("Cannot use sin/cos positional encoding with odd dimension (got dim={:d})".format(D))
    P = torch.zeros(D, H, W)
    # Each dimension use half of D
    D = D // 2
    div_term = torch.exp(torch.arange(0.0, D, 2) * -(math.log(1e4) / D))
    pos_w = torch.arange(0.0, W).unsqueeze(1)
    pos_h = torch.arange(0.0, H).unsqueeze(1)
    P[0:D:2, :, :]  = torch.sin(pos_w * div_term).transpose(0, 1).unsqueeze(1).repeat(1, H, 1)
    P[1:D:2, :, :]  = torch.cos(pos_w * div_term).transpose(0, 1).unsqueeze(1).repeat(1, H, 1)
    P[D::2,  :, :]  = torch.sin(pos_h * div_term).transpose(0, 1).unsqueeze(2).repeat(1, 1, W)
    P[D+1::2,:, :]  = torch.cos(pos_h * div_term).transpose(0, 1).unsqueeze(2).repeat(1, 1, W)
    
    return P